package simulation;

class State extends GlobalSimulation {
	public int numberInQueue = 0, accumulated = 0, noMeasurements = 0;
	private EventList eventList = null;

	State(EventList eventList) {
		this.eventList = eventList;
	}

	// Dispatches the events to the right handler with a java 20 switch statement
	public void TreatEvent(Event event) {
		switch (event.getEventType()) {
			case ARRIVAL -> arrival();
			case READY -> ready();
			case MEASURE -> measure();
		}
		// System.out.println(String.format("Event type: %d, Customers: %d, Time: %.1f",
		// event.getEventType(), numberInQueue, event.getEventTime()));
	}

	// Helper to generate a random number with certain statistical properties
	private static double generateMean(double mean) {
		return 2 * mean * rand.nextDouble();
	}

	private void arrival() {
		if (numberInQueue == 0)
			eventList.insertEvent(new Event(READY, time + generateMean(1)));
		eventList.insertEvent(new Event(ARRIVAL, time + generateMean(2)));
		numberInQueue++;
	}

	private void ready() {
		numberInQueue--;
		if (numberInQueue > 0)
			eventList.insertEvent(new Event(READY, time + generateMean(1)));
	}

	private void measure() {
		accumulated = accumulated + numberInQueue;
		noMeasurements++;
		eventList.insertEvent(new Event(MEASURE, time + generateMean(5)));
	}
}