package simulation;

import java.util.Random;

import org.junit.jupiter.api.Test;

public class EventListTest {
    @Test
    void testEventListConstructor() {
        EventList eventList = new EventList();
        Event event = new Event(1, 0.0);
        eventList.insertEvent(event);
        assert eventList.getHead() == event;
    }

    @Test
    void testOrdering() {
        EventList eventList = new EventList();
        Event event1 = new Event(1, 0.0);
        Event event2 = new Event(1, 1.0);
        Event event3 = new Event(1, 2.0);
        eventList.insertEvent(event3);
        eventList.insertEvent(event2);
        eventList.insertEvent(event1);
        assert eventList.popEvent() == event1;
        assert eventList.popEvent() == event2;
        assert eventList.popEvent() == event3;
        assert eventList.popEvent() == null;
    }

    @Test
    void testLength() {
        Random rand = new Random();
        EventList eventList = new EventList();
        for(int i : rand.ints(0, 100).limit(100).toArray()) {
            eventList.insertEvent(new Event(1, (double) i));
        }
        assert eventList.length() == 100;
    }

    @Test
    void testPop() {
        EventList eventList = new EventList();
        Event event1 = new Event(1, 0.0);
        eventList.insertEvent(event1);
        assert eventList.popEvent() == event1;
        assert eventList.popEvent() == null;
    }

    @Test 
    void testOrderingRandomAmount() {
        Random rand = new Random();
        EventList eventList = new EventList();
        for(int i : rand.ints(0, 100).limit(100).toArray()) {
            eventList.insertEvent(new Event(1, (double) i));
        }

        Event prev = eventList.popEvent();
        for(Event e = eventList.getHead(); e != null; e = eventList.popEvent()) {
            assert prev.getEventTime() <= e.getEventTime();
            prev = e;
        }
    }
}
